##############################################################################
#
# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################
"""Test setup for ZEO connection logic.

The actual tests are in ConnectionTests.py; this file provides the
platform-dependent scaffolding.
"""

# System imports
import os
import time
import socket
import tempfile
import unittest

# Zope/ZODB3 imports
import zLOG

# ZEO test support
from ZEO.tests import forker

# Import the actual test class
from ZEO.tests.ConnectionTests import ConnectionTests

class UnixConnectionTests(ConnectionTests):

    """Add Unix-specific scaffolding to the generic test suite."""

    def startServer(self, create=1, index=0, read_only=0, ro_svr=0,
                    transaction_timeout=None):
        zLOG.LOG("testZEO", zLOG.INFO,
                 "startServer(create=%d, index=%d, read_only=%d, "
                 "transaction_timeout=%s)" %
                 (create, index, read_only, transaction_timeout))
        path = "%s.%d" % (self.file, index)
        addr = self.addr[index]
        pid, server = forker.start_zeo_server(
            self.getStorage(),
            (path, create, read_only), addr, ro_svr,
            transaction_timeout=transaction_timeout)
        self._pids.append(pid)
        self._servers.append(server)

    def shutdownServer(self, index=0):
        zLOG.LOG("testZEO", zLOG.INFO, "shutdownServer(index=%d)" % index)
        self._servers[index].close()
        if self._pids[index] is not None:
            try:
                os.waitpid(self._pids[index], 0)
                self._pids[index] = None
            except os.error, err:
                print err

class FileStorageConnectionTests(UnixConnectionTests):
    def getStorage(self):
        return 'ZODB.FileStorage.FileStorage'

class WindowsConnectionTests(ConnectionTests):

    """Add Windows-specific scaffolding to the generic test suite."""

    def getStorage(self):
        return 'ZODB.FileStorage.FileStorage'

    def startServer(self, create=1, index=0, read_only=0, ro_svr=0,
                    transaction_timeout=None):
        zLOG.LOG("testZEO", zLOG.INFO,
                 "startServer(create=%d, index=%d, read_only=%d, "
                 "transaction_timeout=%s)" %
                 (create, index, read_only, transaction_timeout))
        path = "%s.%d" % (self.file, index)
        addr = self.addr[index]
        args = (path, '='+str(create), '='+str(read_only))
        _addr, test_addr, test_pid = forker.start_zeo_server(
            self.getStorage(),
            args, addr, ro_svr, transaction_timeout)
        self._pids.append(test_pid)
        self._servers.append(test_addr)

    def shutdownServer(self, index=0):
        zLOG.LOG("testZEO", zLOG.INFO, "shutdownServer(index=%d)" % index)
        if self._servers[index] is not None:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            for countdown in xrange(30, -1, -1):
                try:
                    s.connect(self._servers[index])
                    break
                except socket.error:
                    # The server may not be alive yet!
                    if countdown == 0:
                        raise
                    else:
                        time.sleep(0.5) # wait a bit and try again
            s.close()
            self._servers[index] = None
            # XXX waitpid() isn't available until Python 2.3, and this is
            # XXX a serious problem for the tests:  we've simply got no
            # XXX way to know when the server process exits.  So long as
            # XXX the server process is alive, any file it passed to
            # XXX lock_file() remains in the locked state, which can
            # XXX cause the next test (or next phase of the current test)
            # XXX to fail (due to being unable to lock the same file).
            # XXX Berkeley servers seem to take the longest to exit.
            # XXX About all we can do here is sleep, hoping that the server
            # XXX goes away.  1.0 second isn't long enough.  5.0 should be.
            time.sleep(5.0)

class BDBConnectionTests(os.name == "nt" and WindowsConnectionTests
                         or UnixConnectionTests):
    def getStorage(self):
        self.__bdbdir = tempfile.mktemp()
        return 'ZEO.tests.testZEO.BDBFactory'

if os.name == "posix":
    test_classes = [FileStorageConnectionTests]
elif os.name == "nt":
    test_classes = [WindowsConnectionTests]
else:
    raise RuntimeError, "unsupported os: %s" % os.name

import BDBStorage
if BDBStorage.is_available:
    test_classes.append(BDBConnectionTests)


def test_suite():

    # shutup warnings about mktemp
    import warnings
    warnings.filterwarnings("ignore", "mktemp")

    suite = unittest.TestSuite()
    for klass in test_classes:
        sub = unittest.makeSuite(klass, 'check')
        suite.addTest(sub)
    return suite

if __name__ == "__main__":
    unittest.main(defaultTest='test_suite')
